<?php


class Bling {

    public static $baseUrl = 'https://bling.com.br/Api/v2';
    public $integracao = null;
    public $apikey = null;
    public $statusOk = [];
    public $api = null;
    
    public function __construct($id_integracao){
        $this->integracao = Integracao::CarregarObjeto($id_integracao);
        $this->apikey = $this->integracao->get('data1');
        $this->statusOk = explode('#',$this->integracao->get('data3'));
        $this->api = new Api(self::$baseUrl);
    }

    public function getProdutos($inicio='', $fim='', $page=1){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'imagem' => 'S',
            'apikey' => $this->apikey,
        ];
        if(Data::isValid($inicio) && Data::isValid($fim)){
            $query += ['filters' => "dataInclusao[{$inicio} TO {$fim}]"];
        }

        $ret = $this->api->get('/produtos/page='.$page.'/json/?'.http_build_query($query));
        $json = json_decode($ret['response']);

        $retorno = [
            'error' => '',
            'qtd' => 0,
        ];
        if(isset($json->retorno->produtos)){
            $retorno['qtd'] = count($json->retorno->produtos);
            foreach ($json->retorno->produtos as $item) {
                $this->importaProduto($item->produto);
            }
        }else{
            $retorno['error'] = $ret;
        }
        return $retorno;
    }

    public function getPedidos($inicio='', $fim='', $page=1, $cron=false){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'historico' => 'true',
            'apikey' => $this->apikey,
        ];
        if(Data::isValid($inicio) && Data::isValid($fim)){
            $inicio = Data::getFormat($inicio,'d/m/Y');
            $fim    = Data::getFormat($fim,'d/m/Y');
            $query += ['filters' => "dataEmissao[{$inicio} TO {$fim}]"];
        }

        $ret = $this->api->get('/pedidos/page='.$page.'/json/?'.http_build_query($query));
        $json = json_decode($ret['response']);

        $retorno = [
            'error' => '',
            'qtd' => 0,
            'importados' => 0,
        ];
        if(isset($json->retorno->pedidos)){
            foreach ($json->retorno->pedidos as $item) {
                $retorno['qtd'] = count($json->retorno->pedidos);
                if(in_array($item->pedido->situacao, $this->statusOk)){
                    $this->importaPedido($item->pedido);
                    $retorno['importados']++;
                }
            }
        }else{
            $retorno['error'] = $ret;
        }

        if($cron){
            $this->integracao->set('data9', date('Y-m-d'));
            $this->integracao->set('data10', ($retorno['qtd'] < 100 ? $page : $page+1) );
            $this->integracao->set('data11', $page);
            $this->integracao->Alterar();
        }

        return $retorno;
        
    }

    public function getClientes($inicio='', $fim='', $page=1){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'apikey' => $this->apikey,
        ];
        if(Data::isValid($inicio) && Data::isValid($fim)){
            $query += ['filters' => "dataInclusao[{$inicio} TO {$fim}]"];
        }

        $ret = $this->api->get('/contatos/page='.$page.'/json/?'.http_build_query($query));
        $json = json_decode($ret['response']);

        $retorno = [
            'error' => '',
            'qtd' => 0,
        ];
        if(isset($json->retorno->contatos)){
            if(isset($json->retorno->contatos)){
                $retorno['qtd'] = count($json->retorno->contatos);
                foreach ($json->retorno->contatos as $item) {
                    $this->importaCliente($item->contato);
                }
            }
        }else{
            $retorno['error'] = $ret;
        }
        return $retorno;
    }


    public function importaProduto($jobj){
        $rs = Produto::Buscar('id', $jobj->codigo,'sku=');
        if($rs->numRows() == 0){
            $obj = new Produto();
            $obj->set('nome', $jobj->descricao);
            $obj->set('sku', $jobj->codigo);
            $obj->set('descricao', '');
            $obj->set('preco', (float) (isset($jobj->valorunidade) ? $jobj->valorunidade : $jobj->preco) );
            $obj->set('img', '');
            $ret = $obj->Inserir();
            $id = $ret[0];
        }else{
            $rs->next();
            $id = $rs->getInt('id');
        }

        if(isset($jobj->imagem) && count($jobj->imagem) > 0){

            $img = Common::getFileByCurl($jobj->imagem[0]->link);
            file_put_contents($GLOBALS['defaultPath'].'uploads/produtos/produto'.$id.'.jpg', $img);
            $obj = Produto::CarregarObjeto($id);
            $obj->set('img','produto'.$id.'.jpg');
            $obj->Alterar();

        }

        return $id;
    }


    public function importaCliente($jobj){
        $cpf = Common::Replace('/[^0-9]/','', $jobj->cnpj);
        $rs = Cliente::Buscar('id', $cpf,'cpfcnpj=');
        if($rs->numRows() == 0){
            $obj = new Cliente();
            $obj->set('tipo', (strlen($cpf) > 11 ? 1 : 2));
            $obj->set('razaosocial', $jobj->nome);
            $obj->set('nomefantasia', (isset($jobj->fantasia) && $jobj->fantasia != '' ? $jobj->fantasia : $jobj->nome) );
            $obj->set('cpfcnpj', $cpf);
            $obj->set('im', '');
            $obj->set('ie', ($obj->get('tipo') == 2 ? '' : (isset($jobj->ie_rg) ? $jobj->ie_rg : $jobj->ie) ) );
            $obj->set('rg', ($obj->get('tipo') == 2 ? (isset($jobj->ie_rg) ? $jobj->ie_rg : (isset($jobj->rg) ? $jobj->rg : ''))     : '') );
            $obj->set('sexo', 1);
            $obj->set('endereco', $jobj->endereco);
            $obj->set('numero', $jobj->numero);
            $obj->set('complemento', $jobj->complemento);
            $obj->set('bairro', $jobj->bairro);
            $obj->set('id_cidade', 0);
            $obj->set('cidade', $jobj->cidade);
            $obj->set('estado', $jobj->uf);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jobj->cep));
            $obj->set('nasc', (isset($jobj->dataNascimento) && $jobj->dataNascimento != '' && $jobj->dataNascimento != '0000-00-00' ? $jobj->dataNascimento : ''));
            $obj->set('tel', substr(Common::Replace('/[^0-9]/','', $jobj->fone),0,11));
            $obj->set('tel2', substr(Common::Replace('/[^0-9]/','', $jobj->celular),0,11));
            $obj->set('email', $jobj->email);
            $obj->set('obs', '');
            $obj->set('status', 1);
            $obj->set('instagram', '');
            $obj->set('facebook', '');
            $ret = $obj->Inserir();
            return $ret[0];
        }else{
            $rs->next();
            return $rs->getInt('id');
        }
    }

    public function importaPedido($jobj){
        
        $refmp = (isset($jobj->numeroPedidoLoja) ? Common::Replace('/[^0-9]/','',$jobj->numeroPedidoLoja) : '');

        $rs = Pedido::Buscar('id', '','', ($refmp != '' ? "refmp='{$refmp}'" : "ref='{$jobj->numero}'" ));

    
        $canalVenda = (isset($jobj->tipoIntegracao) && $jobj->tipoIntegracao == '' ? $jobj->numeroPedidoLoja : $jobj->tipoIntegracao);
        $id_canalvenda = CanalVenda::load($canalVenda);
        

        if($rs->numRows() == 0){

            $id_formaenvio = 0;
            if(isset($jobj->transporte) && isset($jobj->transporte->servico_correios) ){
                $id_formaenvio = FormaEnvio::load('Correios');
            }else{
                $id_formaenvio = FormaEnvio::load('Transportadora');
            }

            $codigo_rastreio = '';
            if(isset($jobj->codigosRastreamento->codigoRastreamento)){
                $codigo_rastreio = $jobj->codigosRastreamento->codigoRastreamento;
            }

            $data = $jobj->data;
            if(isset($jobj->ocorrencias) && count($jobj->ocorrencias) > 0){
                $data = $jobj->ocorrencias[0]->ocorrencia->dataCriacaoPedido;
            }

            $obj = new Pedido();
            $obj->set('id_empresa', $this->integracao->get('id_empresa'));
            $obj->set('id_cliente', $this->importaCliente($jobj->cliente));
            $obj->set('ref', $jobj->numero);
            $obj->set('refmp', $refmp);
            $obj->set('id_formaenvio', $id_formaenvio);
            $obj->set('frete', number_format(str_replace(',', '', $jobj->valorfrete),2, '.','') );
            $obj->set('data_venda', $data);
            //$obj->set('data_previsao', date('Y-m-d', strtotime($jobj->data . " +15 days")));
            $obj->set('data_previsao', (isset($jobj->dataPrevista) ? $jobj->dataPrevista : '')  );
            $obj->set('data_entrega', '');
            $obj->set('codigo_rastreio', $codigo_rastreio);
            $obj->set('id_canalvenda', $id_canalvenda);
            $jend = (isset($jobj->transporte) ? $jobj->transporte->enderecoEntrega : $jobj->cliente);
            $obj->set('endereco', $jend->endereco);
            $obj->set('numero', $jend->numero);
            $obj->set('complemento', $jend->complemento);
            $obj->set('bairro', $jend->bairro);
            $obj->set('cidade', $jend->cidade);
            $obj->set('estado', $jend->uf);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jend->cep));

            $obj->set('status', 0);
            $ret = $obj->Inserir();
            $id_pedido = $ret[0]+0;

            if($id_pedido > 0){
                foreach ($jobj->itens as $item) {
                    $id_produto = $this->importaProduto($item->item);

                    $objP = new ProdutoPedido();
                    $objP->set('id_pedido', $id_pedido);
                    $objP->set('id_produto', $id_produto);
                    $objP->set('nome', $item->item->descricao);
                    $objP->set('quantidade', (int) $item->item->quantidade);
                    $objP->set('preco', (float) $item->item->valorunidade);
                    $objP->Inserir();
                }
            }
            
            return $id_pedido;
        }else{
            $rs->next();
            $obj = Pedido::CarregarObjeto($rs->getInt('id'));
            $jend = (isset($jobj->transporte) ? $jobj->transporte->enderecoEntrega : $jobj->cliente);
            $obj->set('endereco', $jend->endereco);
            $obj->set('numero', $jend->numero);
            $obj->set('complemento', $jend->complemento);
            $obj->set('bairro', $jend->bairro);
            $obj->set('cidade', $jend->cidade);
            $obj->set('estado', $jend->uf);
            $obj->set('refmp', $refmp);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jend->cep));
            $obj->set('id_canalvenda', $id_canalvenda);
            
            $obj->Alterar();

            return $rs->getInt('id');
        }

    }
}
