<?php


class Entrega {

    public $codigo = '';
    public $api = '';
    public $html = '';
    public $encontrado = false;
    public $entregue = false;
    public $ocorrencia = false;
    public $ultimo_status = '';
    public $data = '';
    public $id_pedido = 0;
    public static $erDevolvido = '/devolvido|devolu|extravi/';
    
    public function __construct($codigo, $id_pedido=0){
        $this->codigo = $codigo;
        $this->id_pedido = $id_pedido;
        $this->reset();
    }

    public function reset(){
        $this->api = '';
        $this->html = '';
        $this->encontrado = false;
        $this->entregue = false;
        $this->ocorrencia = false;
        $this->ultimo_status = '';
        $this->data = '';
    }

    public function updatePedido(){
        if($this->id_pedido > 0){
            $obj = Pedido::CarregarObjeto($this->id_pedido);

            $obj->set('rastreio_api', $this->api);
            $obj->set('rastreio_encontrado', ($this->encontrado?1:0));
            $obj->set('rastreio_msg', trim(substr($this->ultimo_status,0,200)));

            if($this->encontrado){
                if($this->ocorrencia){
                    $obj->set('status',7);

                }elseif($this->entregue){
                    if($obj->get('status') != 2){
                        $obj->set('status',2);
                        $obj->set('data_entrega', $this->data);
                    }
                }elseif($obj->get('status') == 0){
                    $obj->set('status',1);    
                }
                
            }else{
                if(Data::getFormat($obj->get('data'),'U') < strtotime('-2 day') ){
                    $obj->set('rastreio_api', 'error');
                    $obj->set('rastreio_msg', 'Objeto n&atilde;o encontrado em nenhuma api.');
                } else {
                    $obj->set('rastreio_api', '');
                }
            }
            $obj->Alterar();
            return ($this->encontrado);
        }
        return false;
    }

    public function verificaTodos(){
        $executado = false;

        if($this->id_pedido > 0){
            $obj = Pedido::CarregarObjeto($this->id_pedido);
            if($obj->get('rastreio_encontrado') == 1){
                switch ($obj->get('rastreio_api')) {
                    case 'rastrojs': $this->verificaRastroJS(); break;
                    case 'melhorrastreio': $this->verificaMelhorRastreio(); break;
                    case 'mandae': $this->verificaMandae(); break;
                    case 'meli': $this->verificaMeli(); break;
                    default : $this->verificaCorreios(); break;
                }
                $executado = true;   
            }
        }

        if(!$executado){
            $this->verificaCorreios();
            if(!$this->encontrado){
                $this->verificaRastroJS();
                if(!$this->encontrado){
                    $this->verificaMandae();
                    if(!$this->encontrado){
                        $this->verificaMelhorRastreio();
                        if(!$this->encontrado){
                            $this->verificaMeli();
                        }
                    }
                }
            }
        }
        $this->updatePedido();
        return $this->encontrado;
    }

    public function verificaMelhorRastreio(){
        $this->reset();
        $this->api = 'melhorrastreio';
        $url = "https://api.melhorrastreio.com.br/api/v1/trackings/{$this->codigo}";
        $ret = Common::getFileByCurl($url);

        $json = json_decode($ret);

        if(is_object($json) && isset($json->events)){
            $tracks = $json->events;
            $this->encontrado = true;

            if(count($tracks) > 0){
                $last = $tracks[count($tracks)-1];
                $this->entregue = strtolower($last->tag) == 'delivered';
                $this->ocorrencia = preg_match(self::$erDevolvido, strtolower($last->events));
                $this->data = date('Y-m-d',strtotime($tracks[count($tracks)-1]->updated_at));
                $this->ultimo_status = $last->event;
            }
            
            if(count($tracks) > 0){
                $this->html = '
                <table class="table table-hover">
                    <thead>
                      <tr>
                        <th>Data</th>
                        <th>Status</th>
                        <th>Local</th>
                      </tr>
                    </thead>
                    <tbody>';
                $tr = '';
                foreach($tracks as $track){
                    $status = $track->event.'<br><em class="small">'.$track->comment.'</em>';
                    $tr .= '
                    <tr>
                        <td>'.date('d/m/Y H:i:s',strtotime($track->time)).'</td>
                        <td>'.$status.'</td>
                        <td>'.$track->local .'</td>
                     </tr>
                    ';
                }

                $this->html .= $tr.'
                    </tbody>
                </table>
                ';
            }

        }
    }

    public function verificaMandae(){
        $this->reset();
        $this->api = 'mandae';

        $url = "https://rastreamento.api.mandae.com.br/items/{$this->codigo}/tracking?perfil=DESTINATARIO";
        $ret = Common::getFileByCurl($url);
        $json = json_decode($ret);

        if(is_object($json) && !isset($json->error)){
            $this->encontrado = true;
            $this->entregue = (strtolower($json->situacaoEncomenda) == 'entregue');
            $this->ocorrencia = $json->progresso->alerta || $json->progresso->extravio;
            $this->data = date('Y-m-d',strtotime($json->historico[0]->dataHora));

            $tracks = $json->historico;
            if(count($tracks) > 0){
                
                $this->html = '
                <table class="table table-hover">
                    <thead>
                      <tr>
                        <th>Data</th>
                        <th>Status</th>
                        <th>Transportadora</th>
                      </tr>
                    </thead>
                    <tbody>';

                foreach($tracks as $track){
                    $status = (isset($track->descricao) ? $track->descricao.'<br>' : '').'<em class="small">'.$track->complemento.'</em>';
                    if($this->ultimo_status == ''){
                        $this->ultimo_status = (isset($track->descricao) ? $track->descricao : $track->complemento);
                    }
                    $this->html .= '
                    <tr>
                        <td>'.date('d/m/Y H:i:s',strtotime($track->dataHora)).'</td>
                        <td>'.$status.'</td>
                        <td>'.(isset($track->transportadora) ? $track->transportadora : '') .'</td>
                     </tr>
                    ';
                }

                $this->html .= '
                    </tbody>
                </table>
                ';
        
            }
        }
    }

    public function verificaRastroJS(){
        $this->reset();
        $this->api = 'rastrojs';
        $url = "https://rastrojs.herokuapp.com/track/{$this->codigo}/json";
        $ret = Common::getFileByCurl($url);

        $json = json_decode($ret);

        if(is_object($json) && $json->code == 200){
            $this->encontrado = true;
            $this->entregue = $json->data->isDelivered;
            $this->data = date('Y-m-d',strtotime($json->data->updatedAt));
            
            $tracks = $json->data->track;
            if(count($tracks) > 0){
                $last = $tracks[count($tracks)-1];
                $this->ocorrencia = preg_match(self::$erDevolvido, strtolower($last->status));
                $this->ultimo_status = ucfirst($last->status);
            }
            

            if(count($tracks) > 0){
                $this->html = '
                <table class="table table-hover">
                    <thead>
                      <tr>
                        <th>Data</th>
                        <th>Status</th>
                        <th>Unidade</th>
                      </tr>
                    </thead>
                    <tbody>';
                $tr = '';
                foreach($tracks as $track){
                    $tr = '
                    <tr>
                        <td>'.date('d/m/Y H:i:s',strtotime($track->trackedAt)).'</td>
                        <td>'.ucfirst($track->status).'<br><em class="small">'.ucfirst($track->observation).'</em></td>
                        <td>'.ucfirst($track->unit).'</td>
                     </tr>
                    '.$tr;
                }
                $this->html .= $tr.'
                    </tbody>
                </table>
                ';

            }
        }
    }


    public function verificaCorreios(){
        $this->reset();
        $this->api = 'correios';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,"https://www2.correios.com.br/sistemas/rastreamento/resultado_semcontent.cfm");
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS,"objetos={$this->codigo}");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $server_output = curl_exec($ch);
        curl_close ($ch);

        ob_start();
        $listEvent = phpQuery::newDocument(utf8_encode($server_output))->find('.listEvent > tr');
        $eventos = [];
        $data = '';
        if(count($listEvent) > 0){
            $this->html = '
                <table class="table table-hover">
                    <thead>
                      <tr>
                        <th>Data</th>
                        <th>Status</th>
                        <th>Unidade</th>
                      </tr>
                    </thead>
                    <tbody>';

            foreach($listEvent as $event){
                $tds = pq($event)->find('td');
                $arrTd = [];
                foreach($tds as $td){
                    $arrTd[] = trim(preg_replace('/\s\s+/', ' ', $td->textContent));
                }
                $eventos[] = $arrTd;

                $data = trim(substr($arrTd[0], 0, 17));
                $unidade = trim(substr($arrTd[0], 17));
                $status = trim($arrTd[1]);

                if($this->ultimo_status == ''){
                    $this->ultimo_status = $status;
                    $this->ocorrencia = preg_match(self::$erDevolvido, strtolower($status));
                    $pts = explode(' ', $data);
                    $this->data = Data::getFormat($pts[0], 'Y-m-d');
                }

                $this->html .= '
                    <tr>
                        <td>'.$data.'</td>
                        <td>'.$status.'</em></td>
                        <td>'.$unidade.'</td>
                     </tr>
                    ';
                if(preg_match('/objeto entregue/', strtolower($status))) {
                    $this->entregue = true;
                }
            }

            $this->html .= '
                </tbody>
            </table>
            ';
            
            $this->encontrado = true;
            
        }
        ob_end_clean();
    }

    public function verificaMeli(){
        $this->reset();
        $this->api = 'meli';

        $rs = Integracao::Buscar('id', 2, 'sistema');
        if($rs->numRows() > 0) {
            while($rs->next()){
                $obj = new MercadoLivre($rs->getInt('id'));
                $json = $obj->getShipmentHistory($this->codigo);
                
                if($json && !isset($json->error)){

                    if(isset($json->date_history)){
                        $this->encontrado = true;
                        if($json->date_history->date_delivered != ''){
                            $this->entregue = true;
                            $this->data = date('Y-m-d',strtotime($json->date_history->date_delivered));
                        }elseif($json->date_history->date_cancelled != '' || $json->date_history->date_returned != '' || $json->date_history->date_not_delivered != ''){
                            $this->ocorrencia = true;
                        }

                        $this->html = '
                        <table class="table table-hover">
                            <thead>
                              <tr>
                                <th>Data</th>
                                <th>Status</th>
                              </tr>
                            </thead>
                            <tbody>
                                '.($json->date_history->date_created != ''              ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_created)).'</td><td>Pedido criado</td></tr>':'').'
                                '.($json->date_history->date_handling != ''             ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_handling)).'</td><td>Pedido em prepara&ccedil;&atilde;o</td></tr>':'').'
                                '.($json->date_history->date_ready_to_ship != ''        ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_ready_to_ship)).'</td><td>Pedido pronto para envio</td></tr>':'').'
                                '.($json->date_history->date_first_printed != ''        ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_first_printed)).'</td><td>Pedido primeira impress&atilde;o</td></tr>':'').'
                                '.($json->date_history->date_shipped != ''              ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_shipped)).'</td><td>Pedido em transito</td></tr>':'').'
                                '.($json->date_history->date_delivered != ''            ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_delivered)).'</td><td>Pedido entregue</td></tr>':'').'
                                '.($json->date_history->date_cancelled != ''            ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_cancelled)).'</td><td>Pedido cancelado</td></tr>':'').'
                                '.($json->date_history->date_not_delivered != ''        ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_not_delivered)).'</td><td>Pedido n&atilde;o entregue</td></tr>':'').'
                                '.($json->date_history->date_returned != ''             ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_returned)).'</td><td>Pedido retornado</td></tr>':'').'
                                '.($json->date_history->date_delivered_estimated != ''  ? '<tr><td>'.date('d/m/Y H:i:s',strtotime($json->date_history->date_delivered_estimated)).'</td><td>Entrega prevista</td></tr>':'').'
                             </tbody>
                        </table>
                        ';
                    }

                }

                if($this->encontrado){
                    break;
                }
            }
        }

    }

}
