<?php


class Ideris {

    public static $baseUrl = 'http://api.ideris.com.br';
    public $integracao = null;
    public $token = null;
    public $login_token = null;
    public $api = null;
    
    public function __construct($id_integracao){
        $this->integracao = Integracao::CarregarObjeto($id_integracao);
        $this->login_token = $this->integracao->get('data1');
        $this->token = $this->integracao->get('data2');
        $this->api = new Api(self::$baseUrl, ["Content-Type: application/json"]);
    }

    public function login(){
        if($this->login_token == ''){
            throw new Exception('Login is empty', 1);
        }

        $ret = $this->api->post('/Login', ['login_token' => $this->login_token]);
        
        if($ret['error']){
            throw new Exception('Login error: '.$ret['error'], 1);
        }else{
            $this->token = str_replace('"','',$ret['response']);
            $this->integracao->set('data2', $this->token);
            $this->integracao->Alterar();
        }
    }

    public function getProdutos($page=1){
        $limit = 25;
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'offset' => ($page-1)*$limit,
            'limit' => $limit,
        ];

        $ret = $this->api->get('/Produto?'.http_build_query($query), ['Authorization: '.$this->token]);
        
        if( $ret['info']['http_code'] == 401 ) {
            $this->login();
            return $this->getProdutos($page);

        }elseif( $ret['info']['http_code'] == 429 ){
            return [
                'pages' => 0,
                'page' => 0,
                'qtd' => 0,
                'total' => 0,
                'wait' => true,
            ];

        }elseif($ret['error']){
            throw new Exception("Erro importando produtos:" . $ret['error'], 1);
        }

        $json = json_decode($ret['response']);
        if(isset($json->result)){
            foreach ($json->result as $item) {
                $this->importaProduto($item);
            }
        }

        $total  = (isset($json->paging->total) ? $json->paging->total : 0);
        $pages  = ($total > 0 ? ceil($total/$limit) : 0);
        $qtd    = (isset($json->paging->count) ? $json->paging->count : 0);
        
        return [
            'pages' => $pages,
            'page' => $page,
            'qtd' => $qtd,
            'total' => $total,
            'wait' => false,
        ];
        
    }

    public function getPedidos($inicio='', $fim='', $page=1, $cron=false){
        $limit = 20;
        $page = $page+0 < 1 ? 1 : $page;

        $query = [
            'offset' => ($page-1)*$limit,
            'limit' => $limit,
        ];

        if($inicio != '') $query += ['dataInicial' => Data::getFormat($inicio, 'Y-m-d').'T00:00:00-03:00'];
        if($fim != '') $query += ['dataFinal' => Data::getFormat($fim, 'Y-m-d').'T23:59:59-03:00'];

        $ret = $this->api->get('/ListaPedido?'.http_build_query($query), ['Authorization: '.$this->token]);
        
        if( $ret['info']['http_code'] == 401 ) {
            $this->login();
            $this->getPedidos($inicio, $fim, $page);
            
        }elseif( $ret['info']['http_code'] == 429 ){
            return [
                'pages' => 0,
                'page' => 0,
                'qtd' => 0,
                'total' => 0,
                'wait' => true,
                'ids' => [],
            ];

        }elseif($ret['error']){
            throw new Exception("Erro importando pedidos:" . $ret['error'], 1);
        }

        $arrIds = [];
        $json = json_decode($ret['response']);
        if(isset($json->result)){

            $ids = [];
            $qtd = 0;
            foreach ($json->result as $item) {

                if($qtd == 20){
                    $arrIds[] = implode(',',$ids);
                    $ids = [];
                    $qtd=0;
                }
                
                $nmStatus = strtolower($item->status);
                $codigo = Common::Replace('/[^0-9]/','', $item->codigo);
                $status = ($nmStatus == 'entregue' ? 2 : 1);
                
                $rs = Pedido::Buscar('id, status', '','',($codigo != '' ? "refmp='{$codigo}'" : "ref='{$item->id}'"));

                if( $rs->next() ) {
                    $statusAtual = $rs->getInt('status');
                    if($status != $statusAtual){
                        $obj = Pedido::CarregarObjeto($rs->getInt('id'));
                        $obj->set('status', $status);
                        $obj->Alterar();

                    }elseif($statusAtual != 2){
                        $ids[] = $item->id;  
                        $qtd++;  
                    }
                    
                }else{
                    $ids[] = $item->id;
                    $qtd++;
                }
                
            }

            if(count($ids) > 0)
                $arrIds[] = implode(',',$ids);

        }

        if(count($arrIds) > 0){
            foreach ($arrIds as $ids) {
                $this->getPedidosById($ids);
            }
        }

        $total  = (isset($json->paging->total) ? $json->paging->total : 0);
        $pages  = ($total > 0 ? ceil($total/$limit) : 0);
        $qtd    = (isset($json->paging->count) ? $json->paging->count : 0);
    

        if($cron){
            $this->integracao->set('data9', date('Y-m-d'));
            $this->integracao->set('data10', $pages);
            $this->integracao->set('data11', $page);
            $this->integracao->Alterar();
        }

        return [
            'pages' => $pages,
            'page' => $page,
            'qtd' => $qtd,
            'total' => $total,
            'wait' => false,
        ];
        
    }

    public function getPedidosById($ids=''){
        $query = [
            'ids' => $ids,
        ];

        $ret = $this->api->get('/Pedido?'.http_build_query($query), ['Authorization: '.$this->token]);
        
        if( $ret['info']['http_code'] == 401 ) {
            $this->login();
            $this->getPedidosById($ids);
            
        }elseif( $ret['info']['http_code'] == 429 ){
            return [
                'wait' => true,
            ];

        }elseif($ret['error']){
            throw new Exception("Erro importando pedidos:" . $ret['error'], 1);
        }

        $json = json_decode($ret['response']);
        if(isset($json->result)){
            foreach ($json->result as $item) {
                $this->importaPedido($item);
            }
        }

        return [
            'wait' => false,
        ];
        
    }

    public function importaProduto($jobj){
        $sku = '';
        if( isset($jobj->skuProdutoItem) && $jobj->skuProdutoItem != '' ){
            $sku = $jobj->skuProdutoItem;
        } elseif ( isset($jobj->codigoProdutoItem) && $jobj->codigoProdutoItem != '' ){
            $sku = $jobj->codigoProdutoItem;
        } elseif ( isset($jobj->sku) && $jobj->sku != '' ){
            $sku = $jobj->sku;
        } else {
            $sku = $jobj->id;
        }

        $rs = Produto::Buscar('id', $sku,'sku=');
        if($rs->numRows() == 0){
            $obj = new Produto();
            $obj->set('nome', isset($jobj->tituloProdutoItem) ? $jobj->tituloProdutoItem : $jobj->titulo );
            $obj->set('sku', $sku);
            $obj->set('descricao', '');
            $obj->set('preco', (float) (isset($jobj->precoUnitarioItem) ? $jobj->precoUnitarioItem : $jobj->valorVenda) );
            $obj->set('img', '');
            $ret = $obj->Inserir();
            $id = $ret[0];
        }else{
            $rs->next();
            $id = $rs->getInt('id');
        }

        if(isset($jobj->caminhoImagem)){

            $img = Common::getFileByCurl($jobj->caminhoImagem);
            file_put_contents($GLOBALS['defaultPath'].'uploads/produtos/produto'.$id.'.jpg', $img);
            $obj = Produto::CarregarObjeto($id);
            $obj->set('img','produto'.$id.'.jpg');
            $obj->Alterar();

        }

        return $id;
    }


    public function importaCliente($jobj){
        $cpf = Common::Replace('/[^0-9]/','', $jobj->compradorDocumento);
        $rs = Cliente::Buscar('id', $cpf,'cpfcnpj=');
        if($rs->numRows() == 0){
            $obj = new Cliente();
            $nome = trim($jobj->compradorPrimeiroNome.' '.$jobj->compradorSobrenome);

            $estado = 'ND';
            $rs = Estado::Buscar('uf',$jobj->enderecoEntregaEstado,'nome');
            if($rs->next()){
                $estado = $rs->getString('uf');
            }

            $obj->set('tipo', (strlen($cpf) > 11 ? 1 : 2));
            $obj->set('razaosocial', $nome);
            $obj->set('nomefantasia', $nome);
            $obj->set('cpfcnpj', $cpf);
            $obj->set('sexo', 1);
            $obj->set('endereco', $jobj->enderecoEntregaRua);
            $obj->set('numero', $jobj->enderecoEntregaNumero);
            $obj->set('complemento', $jobj->enderecoEntregaComentario);
            $obj->set('bairro', $jobj->enderecoEntregaBairro);
            $obj->set('id_cidade', 0);
            $obj->set('cidade', $jobj->enderecoEntregaCidade);
            $obj->set('estado', $estado);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jobj->enderecoEntregaCep));
            $obj->set('tel', substr(Common::Replace('/[^0-9]/','', $jobj->compradorCodigoAreaTelefone.$jobj->compradorTelefone),0,11));
            $obj->set('email', $jobj->compradorEmail);
            $obj->set('obs', '');
            $obj->set('status', 1);
            $obj->set('instagram', '');
            $obj->set('facebook', '');
            $ret = $obj->Inserir();
            return $ret[0];
        }else{
            $rs->next();
            return $rs->getInt('id');
        }
    }

    public function importaPedido($jobj){
        
        $refmp = Common::Replace('/[^0-9]/','',$jobj->codigo);
        $rs = Pedido::Buscar('id', '','',($refmp != '' ? "refmp='{$refmp}'" : "ref='{$jobj->id}'"));

        $id_canalvenda = CanalVenda::load($jobj->marketplace);

        $estado = 'ND';
        $rsE = Estado::Buscar('uf',$jobj->enderecoEntregaEstado,'nome');
        if($rsE->next()){
            $estado = $rsE->getString('uf');
        }

        if($rs->numRows() == 0){

            $id_formaenvio = FormaEnvio::load('Correios');

            $codigo_rastreio = '';
            if(isset($jobj->numeroRastreio)){
                $codigo_rastreio = $jobj->numeroRastreio;
            }

            $obj = new Pedido();

            $status = 0;
            if(strtolower($jobj->status) == 'entregue')
                $status = 2;
            elseif(strtolower($jobj->status) == 'em transito')
                $status = 1;

            $obj->set('status', $status);
            $obj->set('id_empresa', $this->integracao->get('id_empresa'));
            $obj->set('id_cliente', $this->importaCliente($jobj));
            $obj->set('ref', $jobj->id);
            $obj->set('refmp', $refmp);
            $obj->set('id_formaenvio', $id_formaenvio);
            $obj->set('frete', (float) $jobj->valorTotalComFrete - (float) $jobj->valorTotal);
            $obj->set('data_venda', date('Y-m-d H:i:s',strtotime($jobj->data)) );
            $obj->set('data_previsao', date('Y-m-d', strtotime($jobj->data . " +15 days")));
            $obj->set('data_entrega', '');
            $obj->set('codigo_rastreio', $codigo_rastreio);
            $obj->set('id_canalvenda', $id_canalvenda);
            
            $obj->set('endereco', $jobj->enderecoEntregaRua);
            $obj->set('numero', $jobj->enderecoEntregaNumero);
            $obj->set('complemento', $jobj->enderecoEntregaComentario);
            $obj->set('bairro', $jobj->enderecoEntregaBairro);
            $obj->set('cidade', $jobj->enderecoEntregaCidade);
            $obj->set('estado', $estado);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jobj->enderecoEntregaCep));

            $ret = $obj->Inserir();
            $id_pedido = $ret[0]+0;

            if($id_pedido > 0){
                foreach ($jobj->Item as $item) {
                    $id_produto = $this->importaProduto($item);

                    $objP = new ProdutoPedido();
                    $objP->set('id_pedido', $id_pedido);
                    $objP->set('id_produto', $id_produto);
                    $objP->set('nome', $item->tituloProdutoItem);
                    $objP->set('quantidade', (int) $item->quantidadeItem);
                    $objP->set('preco', (float) $item->precoUnitarioItem);
                    $objP->Inserir();
                }
            }
            
            return $id_pedido;
        }else{
            $rs->next();
            $obj = Pedido::CarregarObjeto($rs->getInt('id'));
            $obj->set('endereco', $jobj->enderecoEntregaRua);
            $obj->set('numero', $jobj->enderecoEntregaNumero);
            $obj->set('complemento', $jobj->enderecoEntregaComentario);
            $obj->set('bairro', $jobj->enderecoEntregaBairro);
            $obj->set('cidade', $jobj->enderecoEntregaCidade);
            $obj->set('estado', $estado);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jobj->enderecoEntregaCep));
            $obj->set('refmp', $refmp);
            $obj->set('id_canalvenda', $id_canalvenda);
            
            $status = $obj->get('status');
            if(strtolower($jobj->status) == 'entregue')
                $status = 2;
            elseif(strtolower($jobj->status) == 'em transito')
                $status = 1;

            $obj->set('status', $status);
            $obj->Alterar();

            return $rs->getInt('id');
        }

    }
}
