<?php


class MercadoLivre {

    public static $baseUrl = 'https://api.mercadolibre.com';
    public $integracao = null;
    public $access_token = '';
    public $refresh_token = '';
    public $user_id = 0;
    public $expiration = 0;
    public $api = null;
    public $meli = null;
    
    public function __construct($id_integracao){
        $this->integracao = Integracao::CarregarObjeto($id_integracao);
        $this->api = new Api(self::$baseUrl);
        $this->meli = new Meli($GLOBALS['MeliId'], $GLOBALS['MeliSecret'], $this->integracao->get('data1'), $this->integracao->get('data2'));
        $this->configureToken();

    }

    public function configureToken(){
        $this->access_token = $this->integracao->get('data1');
        $this->refresh_token = $this->integracao->get('data2');
        $this->user_id = $this->integracao->get('data3');
        $this->expiration = $this->integracao->get('data4')+0;
    }

    public function tokenValid(){
        if(time() > $this->expiration){
            $refresh = $this->meli->refreshAccessToken();
            if(is_object($refresh['body']) && !isset($refresh['body']->error)){
                $this->integracao->set('data1', $refresh['body']->access_token);
                $this->integracao->set('data2', $refresh['body']->refresh_token);
                $this->integracao->set('data3', $refresh['body']->user_id);
                $this->integracao->set('data4', time() + $refresh['body']->expires_in);
                $this->integracao->Alterar();

                $this->configureToken();
                return true;
            }
            return false;
        }else{
            return true;
        }

        
    }

    public function getProdutos($page=1){
        $limit = 50;
        $page = $page < 1 ? 1 : $page;
        $offset = ($page-1)*$limit;
        $retorno = [
            'pages' => 0,
            'page' => $page,
            'qtd' => 0,
            'total' => 0,
            'de' => $offset,
            'ate' => $offset+$limit,
            'error' => true,
        ];

        if($this->tokenValid()){
            $retorno['error'] = false;
        
            $query = [
                'seller_id' => $this->user_id,
                'offset' => $offset,
                'limit' => $limit,
            ];

            $ret = $this->api->get('/sites/MLB/search?'.http_build_query($query));
            $json = json_decode($ret['response']);

            if(is_object($json) && isset($json->results)){

                $retorno['pages'] = ($json->paging->total > 0 ? ceil($json->paging->total / $limit) : 0);
                $retorno['total'] = $json->paging->total;
                foreach ($json->results as $item) {
                    $this->importaProduto($item->id, $item);
                    $retorno['qtd']++;
                }

            }
        }
        return $retorno;
    }

    public function getPedidos($inicio='', $fim='', $page=1, $cron=false){
        $limit = 50;
        $page = $page < 1 ? 1 : $page;
        $offset = ($page-1)*$limit;
        $retorno = [
            'pages' => 0,
            'page' => $page,
            'qtd' => 0,
            'total' => 0,
            'de' => $offset,
            'ate' => $offset+$limit,
            'error' => true,
        ];

        if($this->tokenValid()){
            $retorno['error'] = false;
        
            $query = [
                'seller' => $this->user_id,
                'access_token' => $this->access_token,
                'order.status' => 'paid',
                'offset' => $offset,
                'limit' => $limit,
            ];
            if(Data::isValid($inicio)) $query += ['order.date_created.from' => Data::getFormat($inicio,'Y-m-d').'T00:00:00.000-03:00'];
            if(Data::isValid($fim)) $query += ['&order.date_created.to'=> Data::getFormat($fim,'Y-m-d').'T23:59:59.000-03:00'];
            
            $ret = $this->api->get('/orders/search?'.http_build_query($query));
            $json = json_decode($ret['response']);

            $retorno['pages'] = ($json->paging->total > 0 ? ceil($json->paging->total / $limit) : 0);
            $retorno['total'] = $json->paging->total;
                
            if(is_object($json) && isset($json->results)){
                foreach ($json->results as $item) {
                    $this->importaPedido($item);
                    $retorno['qtd']++;
                }
            }

        }

        if($cron){
            $this->integracao->set('data9', date('Y-m-d'));
            $this->integracao->set('data10', $retorno['pages'] );
            $this->integracao->set('data11', $page);
            $this->integracao->Alterar();
        }

        return $retorno;
    }


    public function importaProduto($id, $json=null){
        if(is_object($json)){
            $jobj = $json;
        }else{
            $ret = $this->api->get('/items?ids='.$id);
            $jobj = json_decode($ret['response']);
            if(isset($jobj[0]) && is_object($jobj[0])){
                $jobj = $jobj[0]->body;
            }elseif(is_object($jobj)){
                $jobj = $jobj->body;
            }else{
                return 0;
            }
        }

        $rs = Produto::Buscar('id', $jobj->id,'sku=');
        if($rs->numRows() == 0){
            $obj = new Produto();
            $obj->set('nome', $jobj->title);
            $obj->set('sku', $jobj->id);
            $obj->set('descricao', '');
            $obj->set('preco', (float) $jobj->price );
            $obj->set('img', '');
            $ret = $obj->Inserir();
            $id = $ret[0];
        }else{
            $rs->next();
            $id = $rs->getInt('id');
        }

        if($jobj->thumbnail != ''){

            $img = Common::getFileByCurl($jobj->thumbnail);
            file_put_contents($GLOBALS['defaultPath'].'uploads/produtos/produto'.$id.'.jpg', $img);
            $obj = Produto::CarregarObjeto($id);
            $obj->set('img','produto'.$id.'.jpg');
            $obj->Alterar();

        }
        
        return $id;
    }


    public function importaCliente($jobj, $addr){
        $cpf = Common::Replace('/[^0-9]/','', $jobj->billing_info->doc_number);
        $rs = Cliente::Buscar('id', $cpf,'cpfcnpj=');
        if($rs->numRows() == 0){
            $obj = new Cliente();
            $obj->set('tipo', (strlen($cpf) > 11 ? 1 : 2));

            $nome = $jobj->first_name.' '.$jobj->last_name;

            $obj->set('razaosocial', $nome);
            $obj->set('nomefantasia', $nome);
            $obj->set('cpfcnpj', $cpf);
            $obj->set('im', '');
            $obj->set('ie', '' );
            $obj->set('rg', '');
            $obj->set('sexo', 1);
            $obj->set('nasc', '');
            $obj->set('tel', substr(Common::Replace('/[^0-9]/','', $jobj->phone->area_code.''.$jobj->phone->number), 0, 18));
            $obj->set('tel', substr(Common::Replace('/[^0-9]/','', $jobj->alternative_phone->area_code.''.$jobj->alternative_phone->number), 0, 18));
            $obj->set('email', $jobj->email);
            $obj->set('obs', '');
            $obj->set('status', 1);
            $obj->set('instagram', '');
            $obj->set('facebook', '');

            if(is_object($addr)){
                $obj->set('endereco', $addr->street_name);
                $obj->set('numero', $addr->street_number);
                $obj->set('complemento', $addr->comment);
                $obj->set('bairro', (isset($addr->neighborhood) ? $addr->neighborhood->name : ''));
                $obj->set('id_cidade', 0);
                $obj->set('cidade', $addr->city->name);
                $obj->set('estado', substr($addr->state->id, 3, 2) );
                $obj->set('cep', $addr->zip_code);
            }
            $ret = $obj->Inserir();
            return $ret[0];
        }else{
            $rs->next();
            return $rs->getInt('id');
        }
    }

    public function importaPedido($jobj){
        
        $refmp = Common::Replace('/[^0-9]/','',$jobj->id);
        $rs = Pedido::Buscar('id', '','',"refmp='{$refmp}'");
        

        if(isset($jobj->shipping->id) && $jobj->shipping->id != ''){
            $json = $this->getShipment($jobj->shipping->id);

        }else{
            $json = null;
        }

        if($rs->numRows() == 0){

            $obj = new Pedido();
            $obj->set('id_formaenvio', 1);
            $obj->set('id_empresa', $this->integracao->get('id_empresa'));
            $obj->set('id_canalvenda', CanalVenda::load('Mercado Livre'));
            $obj->set('ref', $jobj->id);
            $obj->set('refmp', $refmp);
            $obj->set('frete', 0);
            $obj->set('data_venda', date('Y-m-d H:i:s',strtotime($jobj->date_created)));
            $obj->set('data_entrega', '');
            $obj->set('codigo_rastreio', '');
            $obj->set('status', 0);
                
            if(is_object($json) && isset($json->id)){
                $obj->set('id_formaenvio', FormaEnvio::load($json->tracking_method));
                $obj->set('codigo_rastreio', $json->tracking_number);
                $obj->set('frete', $json->shipping_option->cost);
                if($json->status == 'delivered'){
                    $obj->set('status', 2);
                    $obj->set('data_entrega', (isset($json->status_history->date_delivered) && $json->status_history->date_delivered != '' ? date('Y-m-d',strtotime($json->status_history->date_delivered)) : ''));
                }
                if(isset($json->shipping_option->estimated_delivery_time)){
                    $edt = $json->shipping_option->estimated_delivery_time;
                    if($edt->type == 'known_frame'){
                        $obj->set('data_previsao', date('Y-m-d',strtotime($edt->offset->date != '' ? $edt->offset->date : $edt->date)));
                    }elseif($edt->type == 'known'){
                        $obj->set('data_previsao', date('Y-m-d',strtotime($edt->date)) );
                    }
                }

                if(!isset($json->receiver_address)){
                    $addr = $json->receiver_address;
                    if(is_object($addr)){
                        $obj->set('endereco', $addr->street_name);
                        $obj->set('numero', $addr->street_number);
                        $obj->set('complemento', $addr->comment);
                        $obj->set('bairro', (isset($addr->neighborhood) ? $addr->neighborhood->name : ''));
                        $obj->set('cidade', $addr->city->name);
                        $obj->set('estado', substr($addr->state->id, 3, 2) );
                        $obj->set('cep', $addr->zip_code);
                    } 
                }

                $obj->set('id_cliente', $this->importaCliente($jobj->buyer, $json->receiver_address));
            }else{
                $obj->set('id_cliente', $this->importaCliente($jobj->buyer, (!isset($jobj->shipping->receiver_address) ? null : $jobj->shipping->receiver_address)));
            }

            $ret = $obj->Inserir();
            $id_pedido = $ret[0]+0;

            if($id_pedido > 0){
                foreach ($jobj->order_items as $item) {
                    $id_produto = $this->importaProduto($item->item->id);

                    $objP = new ProdutoPedido();
                    $objP->set('id_pedido', $id_pedido);
                    $objP->set('id_produto', $id_produto);
                    $objP->set('nome', $item->item->title);
                    $objP->set('quantidade', (int) $item->quantity);
                    $objP->set('preco', (float) $item->unit_price);
                    $objP->Inserir();
                }
            }
            
            return $id_pedido;
        }else{

            $rs->next();
            $obj = Pedido::CarregarObjeto($rs->getInt('id'));

            if(isset($json->receiver_address)){
                $addr = $json->receiver_address;
                if(is_object($addr)){
                    $obj->set('endereco', $addr->street_name);
                    $obj->set('numero', $addr->street_number);
                    $obj->set('complemento', $addr->comment);
                    $obj->set('bairro', (isset($addr->neighborhood) ? $addr->neighborhood->name : ''));
                    $obj->set('cidade', $addr->city->name);
                    $obj->set('estado', substr($addr->state->id, 3, 2) );
                    $obj->set('cep', $addr->zip_code);
                } 
            }
            
            if(is_object($json) && isset($json->id)){
                if($json->status == 'delivered'){
                    $obj->set('status', 2);
                    $obj->set('data_entrega', (isset($json->status_history->date_delivered) && $json->status_history->date_delivered != '' ? date('Y-m-d',strtotime($json->status_history->date_delivered)) : ''));
                }
            }
            $obj->Alterar();

            $rs2 = ProdutoPedido::Buscar('count(id) qtd', $obj->get('id'),'id_pedido');
            $rs2->next();
            if($rs2->getInt('qtd') == 0 || ProdutoPedido::VerificaExistencia($obj->get('id'),'id_pedido','AND id_produto = 0')){
                ProdutoPedido::ExcluirPorPedido($obj->get('id'));
                foreach ($jobj->order_items as $item) {
                    $id_produto = $this->importaProduto($item->item->id);

                    $objP = new ProdutoPedido();
                    $objP->set('id_pedido', $obj->get('id'));
                    $objP->set('id_produto', $id_produto);
                    $objP->set('nome', $item->item->title);
                    $objP->set('quantidade', (int) $item->quantity);
                    $objP->set('preco', (float) $item->unit_price);
                    $objP->Inserir();
                }
            }

            return $obj->get('id');
        }

    }

    public function getShipment($id){
        if($this->tokenValid()){
            $ret = $this->api->get("/shipments/{$id}?access_token={$this->access_token}");
            return json_decode($ret['response']);
        }
        return null;
    }

    public function getShipmentHistory($id){
        if($this->tokenValid()){
            $ret = $this->api->get("/shipments/{$id}/history?access_token={$this->access_token}");
            return json_decode($ret['response']);
        }
        return null;
    }



}
