<?php


class Tiny {

    public static $baseUrl = 'https://api.tiny.com.br/api2';
    public $integracao = null;
    public $token = null;
    public $statusOk = [];
    public $api = null;
    public static $defaultRet = [
        'pages' => 0,
        'page' => 0,
        'qtd' => 0,
        'ids' => '',
        'error' => false,
        'wait' => false,
    ];
    
    public function __construct($id_integracao){
        $this->integracao = Integracao::CarregarObjeto($id_integracao);
        $this->token = $this->integracao->get('data1');
        $this->statusOk = explode('#',$this->integracao->get('data2'));
        $this->api = new Api(self::$baseUrl);
    }

    public function getProdutos($page=1){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'token' => $this->token,
            'formato' => 'json',
            'pagina' => $page,
        ];

        $ret = $this->api->get('/produtos.pesquisa.php?'.http_build_query($query));
        $json = json_decode($ret['response']);

        $retorno = self::$defaultRet;
        $retorno['pages'] = (isset($json->retorno->numero_paginas) ? $json->retorno->numero_paginas : 1);
        $retorno['page'] = $page;

        if(isset($json->retorno->produtos)){
            $retorno['qtd'] = count($json->retorno->produtos);
            foreach ($json->retorno->produtos as $item) {
                $this->importaProduto($item->produto);
            }
            
        }

        return $retorno;
    }

    public function getClientes($page=1){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'token' => $this->token,
            'formato' => 'json',
            'pagina' => $page,
        ];

        $ret = $this->api->get('/contatos.pesquisa.php?'.http_build_query($query));
        $json = json_decode($ret['response']);

        $retorno = self::$defaultRet;
        $retorno['pages'] = (isset($json->retorno->numero_paginas) ? $json->retorno->numero_paginas : 1);
        $retorno['page'] = $page;

        if(isset($json->retorno->contatos)){
            $retorno['qtd'] = count($json->retorno->contatos);
            foreach ($json->retorno->contatos as $item) {
                $this->importaCliente($item->contato);
            }
            
        }

        return $retorno;
    }

    public function getPedidos($inicio='', $fim='', $page=1, $cron=false){
        $page = $page+0 < 1 ? 1 : $page;
        $query = [
            'token' => $this->token,
            'formato' => 'json',
            'pagina' => $page,
        ];
        if($inicio != '') $query += ['dataInicialOcorrencia' => Data::getFormat($inicio, 'd/m/Y')];
        if($fim != '') $query += ['dataFinalOcorrencia' => Data::getFormat($fim, 'd/m/Y')];

        $ret = $this->api->get('/pedidos.pesquisa.php?'.http_build_query($query));
        
        $json = json_decode($ret['response']);

        $retorno = self::$defaultRet;
        $retorno['pages'] = (isset($json->retorno->numero_paginas) ? $json->retorno->numero_paginas : 1);
        $retorno['page'] = $page;

        $ids = [];
        if(isset($json->retorno->pedidos)){
            $retorno['qtd'] = count($json->retorno->pedidos);
            
            foreach ($json->retorno->pedidos as $item) {
                if(in_array(strtolower($item->pedido->situacao), $this->statusOk)){
                    $ids[] = $item->pedido->id;
                    
                }

            }

            if(count($ids) > 0 ) $retorno['ids'] = implode(',', $ids);
        }


        if($cron){

            if(count($ids) > 0){
                $min = min(20, count($ids));
                for($i = 0; $i < $min; $i++){
                    $this->getPedidoById($ids[$i]);
                }
                array_splice($ids, 0, $min);
                $this->integracao->set('data8', implode(',',$ids));
            }else{
                $this->integracao->set('data8', '');
            }

            $this->integracao->set('data9', date('Y-m-d'));
            $this->integracao->set('data10', $retorno['pages']);
            $this->integracao->set('data11', $page);
            $this->integracao->Alterar();
        }

        return $retorno;
        
    }

    public function getPedidoById($id=''){
        $query = [
            'token' => $this->token,
            'formato' => 'json',
            'id' => $id,
        ];
        
        $ret = $this->api->get('/pedido.obter.php?'.http_build_query($query));
        $json = json_decode($ret['response']);
        //print_r($ret);
        $retorno = self::$defaultRet;
        $retorno['pages'] = (isset($json->retorno->numero_paginas) ? $json->retorno->numero_paginas : 1);
        
        if(isset($json->retorno->pedido)){
            $this->importaPedido($json->retorno->pedido);
            $retorno['qtd']++;
        }
        return $retorno;
    }

    public function importaProduto($jobj){
        $rs = Produto::Buscar('id', $jobj->codigo,'sku=');
        if($rs->numRows() == 0){
            $obj = new Produto();
            $obj->set('nome', (isset($jobj->nome) ? $jobj->nome : $jobj->descricao));
            $obj->set('sku', $jobj->codigo);
            $obj->set('descricao', '');
            $obj->set('preco', (float) (isset($jobj->valor_unitario) ? $jobj->valor_unitario : $jobj->preco) );
            $obj->set('img', '');
            $ret = $obj->Inserir();
            $id = $ret[0];
        }else{
            $rs->next();
            $id = $rs->getInt('id');
        }

        if(isset($jobj->imagem) && count($jobj->imagem) > 0){

            $img = Common::getFileByCurl($jobj->imagem[0]->link);
            file_put_contents($GLOBALS['defaultPath'].'uploads/produtos/produto'.$id.'.jpg', $img);
            $obj = Produto::CarregarObjeto($id);
            $obj->set('img','produto'.$id.'.jpg');
            $obj->Alterar();

        }

        return $id;
    }


    public function importaCliente($jobj){
        $cpf = Common::Replace('/[^0-9]/','', $jobj->cpf_cnpj);
        $rs = Cliente::Buscar('id', $cpf,'cpfcnpj=');
        if($rs->numRows() == 0){
            $obj = new Cliente();
            $obj->set('tipo', (strlen($cpf) > 11 ? 1 : 2));
            $obj->set('razaosocial', $jobj->nome);
            $obj->set('nomefantasia', (isset($jobj->fantasia) && $jobj->fantasia != '' ? $jobj->fantasia : $jobj->nome) );
            $obj->set('cpfcnpj', $cpf);
            $obj->set('im', '');
            $obj->set('ie', (isset($jobj->ie) && $jobj->ie != '' ? $jobj->ie : '') );
            $obj->set('rg', (isset($jobj->rg) && $jobj->rg != '' ? $jobj->rg : '') );
            $obj->set('sexo', 1);
            $obj->set('endereco', $jobj->endereco);
            $obj->set('numero', $jobj->numero);
            $obj->set('complemento', $jobj->complemento);
            $obj->set('bairro', $jobj->bairro);
            $obj->set('id_cidade', 0);
            $obj->set('cidade', $jobj->cidade);
            $obj->set('estado', $jobj->uf);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jobj->cep));
            $obj->set('tel', substr(Common::Replace('/[^0-9]/','', $jobj->fone),0,11));
            $obj->set('email', $jobj->email);
            $obj->set('obs', '');
            $obj->set('status', 1);
            $obj->set('instagram', '');
            $obj->set('facebook', '');
            $ret = $obj->Inserir();
            return $ret[0];
        }else{
            $rs->next();
            return $rs->getInt('id');
        }
    }

    public function importaPedido($jobj){
        
        $pedidoEcommerce = $jobj->numero_ecommerce;
        $canalVenda = '';
        if(isset($jobj->ecommerce)){
            $canalVenda = isset($jobj->ecommerce->canalVenda) ? $jobj->ecommerce->canalVenda : $jobj->ecommerce->numeroPedidoEcommerce;
            if($jobj->ecommerce->numeroPedidoEcommerce){
                $pedidoEcommerce = $jobj->ecommerce->numeroPedidoEcommerce;
            }
        }
        $id_canalvenda = CanalVenda::load($canalVenda);

        $refmp = Common::Replace('/[^0-9]/','',$pedidoEcommerce);

        $rs = Pedido::Buscar('id', '','', ($refmp != '' ? "refmp='{$refmp}'" : "ref='{$jobj->id}'" ));
        //$rs = Pedido::Buscar('id', '','',"ref='{$jobj->id}'");

        if($rs->numRows() == 0){

            $id_formaenvio = 0;
            if(isset($jobj->forma_frete) && $jobj->forma_frete != '' ){
                $id_formaenvio = FormaEnvio::load($jobj->forma_frete);
            }

            $codigo_rastreio = '';
            if(isset($jobj->codigo_rastreamento)){
                $codigo_rastreio = $jobj->codigo_rastreamento;
            }

            $obj = new Pedido();
            $obj->set('id_empresa', $this->integracao->get('id_empresa'));
            $obj->set('id_cliente', $this->importaCliente($jobj->cliente));
            $obj->set('ref', $jobj->id);
            $obj->set('refmp', $refmp);
            $obj->set('id_formaenvio', $id_formaenvio);
            $obj->set('frete', number_format(str_replace(',', '', $jobj->valor_frete),2,'.',''));
            $obj->set('data_venda', Data::getFormat($jobj->data_pedido,'Y-m-d'));
            //$obj->set('data_previsao', date('Y-m-d', strtotime($jobj->data . " +15 days")));
            $obj->set('data_previsao', Data::isValid($jobj->data_prevista) ? Data::getFormat($jobj->data_prevista,'Y-m-d') : '' );
            $obj->set('data_entrega', '');
            $obj->set('codigo_rastreio', $codigo_rastreio);
            $obj->set('id_canalvenda', $id_canalvenda);
            $jend = $jobj->cliente;
            $obj->set('endereco', $jend->endereco);
            $obj->set('numero', $jend->numero);
            $obj->set('complemento', $jend->complemento);
            $obj->set('bairro', $jend->bairro);
            $obj->set('cidade', $jend->cidade);
            $obj->set('estado', $jend->uf);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jend->cep));

            $obj->set('status', 0);
            $ret = $obj->Inserir();
            $id_pedido = $ret[0]+0;

            if($id_pedido > 0){
                foreach ($jobj->itens as $item) {
                    $id_produto = $this->importaProduto($item->item);

                    $objP = new ProdutoPedido();
                    $objP->set('id_pedido', $id_pedido);
                    $objP->set('id_produto', $id_produto);
                    $objP->set('nome', $item->item->descricao);
                    $objP->set('quantidade', (int) $item->item->quantidade);
                    $objP->set('preco', (float) $item->item->valor_unitario);
                    $objP->Inserir();
                }
            }
            
            return $id_pedido;
        }else{
            $rs->next();
            $obj = Pedido::CarregarObjeto($rs->getInt('id'));
            $jend = $jobj->cliente;
            $obj->set('endereco', $jend->endereco);
            $obj->set('numero', $jend->numero);
            $obj->set('complemento', $jend->complemento);
            $obj->set('bairro', $jend->bairro);
            $obj->set('cidade', $jend->cidade);
            $obj->set('estado', $jend->uf);
            $obj->set('refmp', $refmp);
            $obj->set('cep', Common::Replace('/[^0-9]/','', $jend->cep));
            $obj->set('id_canalvenda', $id_canalvenda);
            
            $obj->Alterar();

            return $rs->getInt('id');
        }
    }
}
